@@ -1,9 +1,9 @@
-/************************************************************************
- * This program is Copyright (C) 1986-1996 by Jonathan Payne.  JOVE is  *
- * provided to you without charge, and with no warranty.  You may give  *
- * away copies of JOVE, including sources, provided that this notice is *
- * included in all the files.                                           *
- ************************************************************************/
+/**************************************************************************
+ * This program is Copyright (C) 1986-2002 by Jonathan Payne.  JOVE is    *
+ * provided by Jonathan and Jovehacks without charge and without          *
+ * warranty.  You may copy, modify, and/or distribute JOVE, provided that *
+ * this notice is included in all the source files and documentation.     *
+ **************************************************************************/
 
 /* Contains the main loop initializations, and some system dependent
    type things, e.g. putting terminal in CBREAK mode, etc. */
@@ -132,7 +132,7 @@ private volatile bool	InWaitChar = NO;
 /*ARGSUSED*/
 private SIGRESTYPE
 AlarmHandler(junk)
-int	junk;	/* passed in on signal; of no interest */
+int	UNUSED(junk);	/* passed in on signal; of no interest */
 {
 	int save_errno = errno;	/* Subtle, but necessary! */
 
@@ -223,9 +223,6 @@ private char	LibDir[FILESIZE] = LIBDIR;
  *   (it cannot be one under UNIX).  This is the only code
  *   for which buffers are not saved.
  *
- * - SIGINT is caused by the user hitting the INTR key.
- *   We give him a choice of death or continuation.
- *
  * - SIGHUP is caused by loss of connection.  This code and
  *   code 0 are the only ones which don't cause an abort.
  *   Generated by OS and by JOVE itself.
@@ -238,62 +235,6 @@ int	code;
 	int save_errno = errno;	/* Subtle, but necessary! */
 	bool	DelTmps = YES;		/* Usually we delete them. */
 
-	if (code == SIGINT) {
-		char	c;
-#ifdef WIN32
-		c = FatalErrorMessage("Fatal interrupt encountered. Abort?");
-#else /* !WIN32 */
-# ifdef PIPEPROCS
-		bool	started;
-# endif
-
-		resetsighandler(SIGINT, finish);
-		f_mess("Abort (Type 'n' if you're not sure)? ");
-		Placur(ILI, min(CO - 2, calc_pos(mesgbuf, MAXCOLS)));
-		flushscreen();
-# ifdef UNIX
-#  ifdef PIPEPROCS
-		started = kbd_stop();
-#  endif
-		/*
-		 * Yuk!  This doesn't deal with all cases, we really need a
-		 * standard jove input routine that's lower than kbd_getch so
-		 * that this can use it.  The code that this replaces was even
-		 * more ugly.  What about nonblocking reads? -- MM.
-		 */
-#  ifdef NONBLOCKINGREAD
-		setblock(YES);	/* turn blocking on (in case it was off) */
-#  endif
-		for (;;) {
-			c = 'n';
-			if (read(0, (UnivPtr) &c, sizeof(c)) < 0) {
-				switch (errno) {
-				case EINTR:
-#  if EWOULDBLOCK != EAGAIN	/* aliases in System Vr4 */
-				case EWOULDBLOCK:
-#  endif
-				case EAGAIN:
-					continue;
-				}
-			}
-			break;
-		}
-#  ifdef PIPEPROCS
-		if (started)
-			kbd_strt();
-#  endif /* PIPEPROCS */
-# endif /* UNIX */
-# ifdef MSDOS
-		c = getrawinchar();
-# endif /* MSDOS */
-		message(NullStr);
-#endif /* !WIN32 */
-		if (c != 'y') {
-			redisplay();
-			errno = save_errno;
-			return SIGRESVALUE;
-		}
-	}
 	DisabledRedisplay = YES;
 #ifndef MAC
 	UnsetTerm(NO);
@@ -344,14 +285,79 @@ int	code;
 	/*NOTREACHED*/
 }
 
+/* SIGINT is caused by the user hitting the INTR key.
+ * We give him a choice of death or continuation.
+ */
+private SIGRESTYPE
+handle_sigint(code)
+int	code;
+{
+	int save_errno = errno;	/* Subtle, but necessary! */
+	char	c;
+#ifdef WIN32
+	c = FatalErrorMessage("Fatal interrupt encountered. Abort?");
+#else /* !WIN32 */
+# ifdef PIPEPROCS
+	bool	started;
+# endif
+
+	resetsighandler(SIGINT, handle_sigint);
+	f_mess("Abort (Type 'n' if you're not sure)? ");
+	Placur(ILI, min(CO - 2, calc_pos(mesgbuf, MAXCOLS)));
+	flushscreen();
+# ifdef UNIX
+#  ifdef PIPEPROCS
+	started = kbd_stop();
+#  endif
+	/*
+	 * Yuk!  This doesn't deal with all cases, we really need a
+	 * standard jove input routine that's lower than kbd_getch so
+	 * that this can use it.  The code that this replaces was even
+	 * more ugly.  What about nonblocking reads? -- MM.
+	 */
+#  ifdef NONBLOCKINGREAD
+	setblock(YES);	/* turn blocking on (in case it was off) */
+#  endif
+	for (;;) {
+		c = 'n';
+		if (read(0, (UnivPtr) &c, sizeof(c)) < 0) {
+			switch (errno) {
+			case EINTR:
+#  if EWOULDBLOCK != EAGAIN	/* aliases in System Vr4 */
+			case EWOULDBLOCK:
+#  endif
+			case EAGAIN:
+				continue;
+			}
+		}
+		break;
+	}
+#  ifdef PIPEPROCS
+	if (started)
+		kbd_strt();
+#  endif /* PIPEPROCS */
+# endif /* UNIX */
+# ifdef MSDOS
+	c = getrawinchar();
+# endif /* MSDOS */
+	message(NullStr);
+#endif /* !WIN32 */
+	if (c == 'y') {
+		errno = save_errno;
+		finish(code);
+		/* NOTREACHED */
+	}
+	redisplay();
+	errno = save_errno;
+	return SIGRESVALUE;
+}
+
 private char	smbuf[20],
 		*bp = smbuf;
 private int	nchars = 0;
 
 #ifdef NONBLOCKINGREAD
 
-# include <fcntl.h>
-
 private void
 setblock(on)	/* turn blocking on or off */
 bool	on;
@@ -546,8 +552,8 @@ kbd_getch()
 		}
 		c = ZXRC(*bp++);
 #if !defined(PCNONASCII) && !defined(MAC)	/* if not done elsewhere */
-		if ((c & 0200) && MetaKey) {
-			*--bp = c & ~0200;
+		if ((c & METABIT) && MetaKey) {
+			*--bp = c & ~METABIT;
 			nchars += 1;
 			c = ESC;
 		}
@@ -566,6 +572,7 @@ charp()
 {
 	if (InJoverc != 0 || kbdpeek != EOF || nchars > 0 || Inputp != NULL)
 		return InputPending = YES;
+
 #ifdef FIONREAD
 	{
 		/*
@@ -731,6 +738,7 @@ int	delay;
 	for (;;)  {
 		if (charp())
 			break;
+
 		_dos_gettime(&tc);
 		now = tc.second * 100 + tc.hsecond;
 		if (now < start)
@@ -976,7 +984,8 @@ Push()
 # ifdef MSDOS_PROCS
 #  ifdef MSDOS
 	UnsetTerm(YES);
-	if (spawnl(0, Shell, basename(Shell), (char *)NULL) == -1)
+	/* Zortech's prototype for the third parameter to spawnl is missing "const" */
+	if (spawnl(0, Shell, (char *)jbasename(Shell), (char *)NULL) == -1)
 		s_mess("[Spawn failed %d]", errno);
 	SetTerm();
 #   ifdef WINRESIZE
@@ -1023,7 +1032,7 @@ Push()
 		(void) setsighandler(SIGINT, SIG_DFL);
 		jcloseall();
 		/* note that curbuf->bfname may be NULL */
-		execl(Shell, basename(Shell), "-is", pr_name(curbuf->b_fname, NO),
+		execl(Shell, jbasename(Shell), "-is", pr_name(curbuf->b_fname, NO),
 			(char *)NULL);
 		raw_complain("[Execl failed: %s]", strerror(errno));
 		_exit(1);
@@ -1084,12 +1093,14 @@ getch()
 	if (Inputp != NULL) {
 		if ((c = ZXC(*Inputp++)) != '\0')
 			return LastKeyStruck = c;
+
 		Inputp = NULL;
 	}
 
 	if (InJoverc) {
 		/* somethings wrong if Inputp runs out while
-		   we're reading a .joverc file. */
+		 * we're reading a .joverc file.
+		 */
 		complain("[command line too short]");
 	}
 
@@ -1134,7 +1145,7 @@ dorecover()
 # ifdef PIPEPROCS
 	kbd_kill();		/* kill the keyboard process */
 # endif
-	swritef(Recover, sizeof(Recover), "%s/recover", LibDir);
+	PathCat(Recover, sizeof(Recover), LibDir, "recover");
 	execl(Recover, "recover", "-d", TmpDir, (char *) NULL);
 	writef("%s: execl failed! %s\n", Recover, strerror(errno));
 	flushscreen();
@@ -1157,7 +1168,6 @@ char	*argv[];
 	int	lineno = 0,
 		nwinds = 1;
 	char	*pattern = NULL;
-	Buffer	*b;
 
 	while (argc > 1) {
 		switch (argv[1][0]) {
@@ -1236,14 +1246,18 @@ char	*argv[];
 			}
 			break;
 		default:
-			{
-			bool	force = (nwinds > 0 || lineno != 0 || pattern != NULL);
+			/* Process a file argument.
+			 * We arrange that the last file that has a window
+			 * becomes the current buffer and that the last file
+			 * without a window (if any) becomes the alternate buffer.
+			 */
+			minib_add(argv[1], nwinds > 0);
+			if (nwinds > 0 || lineno != 0 || pattern != NULL) {
+				Buffer
+					*prevbuf = curbuf,
+					*b = do_find(curwind, argv[1], YES, YES);
 
-			minib_add(argv[1], force);
-			b = do_find(nwinds > 0 ? curwind : (Window *) NULL,
-				    argv[1], force, YES);
-			if (force) {
-				SetABuf(curbuf);
+				SetABuf(prevbuf);
 				SetBuf(b);
 				if (lineno > 0)
 					SetLine(next_line(curbuf->b_first, lineno - 1));
@@ -1258,14 +1272,28 @@ char	*argv[];
 					else
 						complain("Couldn't match pattern in file.");
 				}
-				if (nwinds > 1)
-					NextWindow();
-				if (nwinds > 0)
+				if (nwinds > 0) {
 					nwinds -= 1;
+					if (nwinds > 0)
+						NextWindow();
+				} else {
+					/* We only borrowed the window:
+					 * let's restore the rightful tennant.
+					 * As a consolation, make this buffer the alternate.
+					 */
+					SetABuf(b);
+					SetBuf(prevbuf);
+					tiewind(curwind, prevbuf);
+				}
+			} else {
+				/* There is no need to actually read the file.
+				 * find the file, but don't force it, or select the buffer.
+				 * As a consolation, make this buffer the alternate.
+				 */
+				SetABuf(do_find((Window *)NULL, argv[1], NO, YES));
 			}
 			lineno = 0;
 			pattern = NULL;
-			}
 			break;
 		}
 		argv += 1;
@@ -1331,12 +1359,20 @@ raw_complain(fmt, va_alist)
 {
 	char	buf[MESG_SIZE];
 	va_list	ap;
+	UnivConstPtr	bp;
+	size_t	rem;
+	ssize_t r;
 
 	va_init(ap, fmt);
 	format(buf, sizeof(buf) - 2, fmt, ap);
 	va_end(ap);
 	strcat(buf, "\r\n");	/* \r *may* be redundant */
-	(void) write(2, (UnivConstPtr)buf, strlen(buf));
+	for (bp = buf, rem = strlen(buf)
+	; rem > 0 && (r = write(2, bp, rem)) != rem
+	; bp += r, rem -=r) {
+		if (r < 0 || errno != EINTR)
+		    break;	/* give up */
+	}
 }
 
 #ifdef STDARGS
@@ -1387,6 +1423,8 @@ Recur()
 	DelMark(m);
 }
 
+bool	SaveOnExit = NO;	/* VAR: offer to save buffers on exit */
+
 private int	iniargc;	/* main sets these for DoKeys() */
 private char	**iniargv;
 
@@ -1411,9 +1449,12 @@ bool	firsttime;
 				if (!yes_or_no_p("Some MACROS haven't been saved; leave anyway? "))
 					break;
 			}
+			/* Maybe should offer to save macros too ? */
 			if (ModBufs(NO)) {
 				rbell();
-				if (!yes_or_no_p("Some buffers haven't been saved; leave anyway? "))
+				if (SaveOnExit)
+					put_bufs(YES);
+				else if (!yes_or_no_p("Some buffers haven't been saved; leave anyway? "))
 					break;
 			}
 #ifdef IPROCS
@@ -1465,6 +1506,7 @@ register char	**args,
 	while (*args) {
 		if (strcmp(*args, str) == 0)
 			return args;
+
 		args += 1;
 	}
 	return NULL;
@@ -1474,7 +1516,7 @@ register char	**args,
 /*ARGSUSED*/
 SIGRESTYPE
 win_reshape(junk)
-int	junk;	/* passed in when invoked by a signal; of no interest */
+int	UNUSED(junk);	/* passed in when invoked by a signal; of no interest */
 {
 	int save_errno = errno;	/* Subtle, but necessary! */
 
@@ -1503,7 +1545,7 @@ char	*mess;
 bool	raw;
 {
 	if (from != NULL) {
-		char	*ugh;
+		const char	*ugh;
 
 		if (strlen(from) >= maxsize)
 			ugh = "too long";
@@ -1523,6 +1565,9 @@ bool	raw;
 	return NO;
 }
 
+#define envcpy(buf, varname) \
+	carefulcpy(buf, getenv(varname), sizeof(buf), varname, NO)
+
 private void
 dojovercs(dosys, dousr)
 bool	dosys;
@@ -1531,7 +1576,7 @@ bool	dousr;
 	char	Joverc[FILESIZE];
 
 	if (dosys) {
-		swritef(Joverc, sizeof(Joverc), "%s/jove.rc", ShareDir);
+		PathCat(Joverc, sizeof(Joverc), ShareDir, "jove.rc");
 		(void) joverc(Joverc);	/* system wide jove.rc */
 	}
 
@@ -1539,11 +1584,11 @@ bool	dousr;
 #ifdef MSFILESYSTEM
 		/* We don't want to run the same rc file twice */
 		if (!dosys || strcmp(HomeDir, ShareDir) != 0) {
-			swritef(Joverc, sizeof(Joverc), "%s/jove.rc", HomeDir);
+			PathCat(Joverc, sizeof(Joverc), HomeDir, "jove.rc");
 			(void) joverc(Joverc);	/* jove.rc in home directory */
 		}
 #else
-		swritef(Joverc, sizeof(Joverc), "%s/.joverc", HomeDir);
+		PathCat(Joverc, sizeof(Joverc), HomeDir, ".joverc");
 		(void) joverc(Joverc);	/* .joverc in home directory */
 #endif
 	}
@@ -1650,9 +1695,11 @@ char	*argv[];
 		char *homepath = getenv("HOMEPATH");
 
 		if (homedrive != NULL && homepath != NULL) {
-			HomeDir = emalloc(strlen(homedrive) + strlen(homepath) + 1);
-			strcpy(HomeDir, homedrive);
-			strcat(HomeDir, homepath);
+			char *p = emalloc(strlen(homedrive) + strlen(homepath) + 1);
+
+			strcpy(p, homedrive);
+			strcat(p, homepath);
+			HomeDir = p;
 		} else {
 			HomeDir = copystr(pwd());
 		}
@@ -1700,7 +1747,7 @@ char	*argv[];
 			if (!carefulcpy(LibDir, lo, sizeof(LibDir)-9, "LibDir", YES))
 				finish(0);
 #  ifdef PIPEPROCS
-		swritef(Portsrv, sizeof(Portsrv), "%s/portsrv", LibDir);
+		PathCat(Portsrv, sizeof(Portsrv), LibDir, "portsrv");
 #  endif
 # endif /* NEED_LIBDIR */
 	}
@@ -1709,13 +1756,14 @@ char	*argv[];
 	ShowVersion();	/* but the 'carefulcpy's which follow might overwrite it */
 
 	/* import the temporary file path from the environment
-	   and fix the string, so that we can append a slash
-	   safely	*/
+	 * and fix the string, so that we can append a slash
+	 * safely
+	 */
 #ifdef MSFILESYSTEM
-	carefulcpy(TmpDir, getenv("TEMP"), sizeof(TmpDir), "TEMP", NO);
+	envcpy(TmpDir, "TEMP");
 #endif
 #ifndef MAC	/* no environment in MacOS */
-	carefulcpy(TmpDir, getenv("TMPDIR"), sizeof(TmpDir), "TMPDIR", NO);
+	envcpy(TmpDir, "TMPDIR");
 #endif
 	{
 		char	*cp = &TmpDir[strlen(TmpDir)];
@@ -1730,14 +1778,14 @@ char	*argv[];
 
 #ifdef SUBSHELL
 # ifdef MSFILESYSTEM	/* ??? Is this the right test? */
-	carefulcpy(Shell, getenv("COMSPEC"), sizeof(Shell), "COMSPEC", NO);
+	envcpy(Shell, "COMSPEC");
 	/* SHELL, if present in DOS environment, will take precedence over COMSPEC */
 # endif /* MSFILESYSTEM */
-	carefulcpy(Shell, getenv("SHELL"), sizeof(Shell), "SHELL", NO);
+	envcpy(Shell, "SHELL");
 #endif /* SUBSHELL */
 
 #ifdef UNIX
-	carefulcpy(Mailbox, getenv("MAIL"), sizeof(Mailbox), "MAIL", NO);
+	envcpy(Mailbox, "MAIL");
 #endif
 
 	dojovercs(scanvec(argv, "-J") == NULL, scanvec(argv, "-j") == NULL);
@@ -1750,7 +1798,7 @@ char	*argv[];
 #ifdef UNIX
 # ifndef DEBUGCRASH
 	(void) setsighandler(SIGHUP, finish);
-	(void) setsighandler(SIGINT, finish);
+	(void) setsighandler(SIGINT, handle_sigint);
 #  ifdef SIGBUS
 	(void) setsighandler(SIGBUS, finish);
 #  endif /* SIGBUS */
@@ -1767,5 +1815,5 @@ char	*argv[];
 	RedrawDisplay();	/* start the redisplay process. */
 	DoKeys(YES);
 	finish(0);
-	/* NOTREACHED*/
+	/* NOTREACHED */
 }
