@@ -189,7 +189,7 @@ for (fnp = fnt; fnp->in != NULL; fnp++) {
 			sp->first = ch;
 			sp->last = -1;
 			strcpy(sp->condition, line);
-			fprintf(of, line);
+			fputs(line, of);
 		} else if (StartsWith(line, "#else")) {
 			if (sp == stackspace || sp->last != -1) {
 				fprintf(stderr, "ifdef/endif mismatch at line %d!\n",
@@ -198,7 +198,7 @@ for (fnp = fnt; fnp->in != NULL; fnp++) {
 			}
 			sp->last = ch;
 			ch = sp->first;
-			fprintf(of, line);
+			fputs(line, of);
 		} else if (StartsWith(line, "#endif")) {
 			if (sp == stackspace) {
 				fprintf(stderr, "ifdef/endif mismatch at line %d!\n",
@@ -211,7 +211,7 @@ for (fnp = fnt; fnp->in != NULL; fnp++) {
 					lino);
 			}
 			sp -= 1;
-			fprintf(of, line);
+			fputs(line, of);
 #ifdef MAC
 		} else if (StartsWith(line, "#MENU")) {
 			inmenu = YES;
@@ -260,7 +260,7 @@ for (fnp = fnt; fnp->in != NULL; fnp++) {
 			}
 		} else {
 			/* If unrecognized, pass and prepare to start new table */
-			fprintf(of, line);
+			fputs(line, of);
 			ch = 0;
 		}
 	}
