@@ -646,7 +646,7 @@
 #  define NBUF		3
 # endif
 # ifndef JLGBUFSIZ
-#  define JLGBUFSIZ	9
+#  define JLGBUFSIZ	9	/* 512kB is the traditional UNIX block size */
 # endif
 #endif
 
@@ -655,7 +655,7 @@
 #endif
 
 #ifndef JLGBUFSIZ
-# define JLGBUFSIZ 10
+# define JLGBUFSIZ 12	/* perhaps a good match for 4kB pages */
 #endif
 
 #define JBUFSIZ	(1<<JLGBUFSIZ)
